{ :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  :: QuickReport 3.0 for Delphi and C++Builder               ::
  ::                                                         ::
  :: RoundRect component for QuickReport 3                   ::
  ::                                                         ::
  :: Copyright (c) 1999 QuSoft AS                            ::
  :: All Rights Reserved                                     ::
  ::                                                         ::
  :: web: http://www.qusoft.com                              ::
  ::                                                         ::
  ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: }

unit qrRRect;

interface

uses
  Windows, SysUtils, Classes, Graphics, printers, QRPrntr, Quickrpt, qrctrls;

{$R-}
{$B-}

type
  { TQRRoundRect }
  TQRRoundRect = class(TQRPrintable)
  private
    FBrush : TBrush;
    FPen : TPen;
    FEllipseHeight: extended;
    FEllipseWidth: extended;
    procedure SetBrush(Value : TBrush);
    procedure SetPen(Value : TPen);
    procedure SetEllipse(index: integer; val: extended);
  protected
    procedure Paint; override;
    procedure Print(OfsX, OfsY : integer); override;
    procedure StyleChanged(sender : TObject);
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
  published
    property Brush : TBrush read FBrush write SetBrush;
    property Height default 65;
    property Pen : TPen read FPen write Setpen;
    property Width default 65;
    property EllipseHeight: extended index 0 read FEllipseHeight write SetEllipse;
    property EllipseWidth: extended index 1 read FEllipseWidth write SetEllipse;
  end;

procedure Register;

implementation

constructor TQRRoundRect.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  width := 65;
  Height := 65;
  FEllipseHeight := 40.0;
  FEllipseWidth := 40.0;

  FPen := TPen.Create;
  FBrush := TBrush.Create;
  FBrush.OnChange := StyleChanged;
  FPen.OnChange := StyleChanged;
end;

procedure TQRRoundRect.SetEllipse(index: integer; val: extended);
begin
  if index = 0 then
    FEllipseHeight := val
  else
    FEllipseWidth := val;
  invalidate;
end;

procedure TQRRoundRect.StyleChanged(Sender : TObject);
begin
  invalidate;
end;

procedure TQRRoundRect.SetBrush(Value : TBrush);
begin
  FBrush.Assign(Value);
end;

procedure TQRRoundRect.SetPen(Value : TPen);
begin
  FPen.Assign(value);
end;

procedure TQRRoundRect.Paint;
begin
  inherited paint;
  with Canvas do
  begin
    Pen := FPen;
    Brush := FBrush;
    // For an explanation of the RoundRect() parameters, please refer
    // to the Win32 SDK help file.
    RoundRect(0,0,Width,Height,trunc(EllipseWidth),trunc(EllipseHeight));
  end;
end;

procedure TQRRoundRect.Print(OfsX,OfsY : Integer);
var
  EW, EH: extended;
begin
  if ParentReport.FinalPass and IsEnabled then
  begin
    QRPrinter.Canvas.Brush := Brush;
    QRPrinter.Canvas.Pen := Pen;
    // We need to convert the EllipseWidth & EllipseHeight
    // properties to the canvas units.  The Size property has this
    // built in, but we can not access those conversion functions
    // outside of the quickrep unit. So we cheat by calculating the
    // conversion factor from the two different values.
    EW := (EllipseWidth * 1.0) * (Size.Width / (Width * 1.0));
    EH := (EllipseHeight * 1.0) * (Size.Height / (Height * 1.0));
    with QRPrinter do
      with Canvas do
      begin
        RoundRect(XPos(OfsX + Size.Left), YPos(OfsY + Size.Top),
          XPos(OfsX+Size.Left + Size.Width), YPos(OfsY + Size.Top + Size.Height),
          XSize(EW), YSize(EH));
      end;
  end;
end;

destructor TQRRoundRect.Destroy;
begin
  FPen.Free;
  FBrush.Free;
  inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('QReport', [TQRRoundRect]);
end;

end.
